#!/bin/bash
#
# Versione 1.0 - 2/12/2020
#
###
###   converter.sh  Converte file .MD con un header Hugo (vedi $HEADER).
###                 Aggiunge a tutte le frasi il shortcode "p" e rimuove
###                 linee vuote con blank.
###
### Usage:
###   converter.sh  <input> [<output>]
###                 Se non viene specificato alcun output, il file
###                 sorgente verrà sovrascritto.
###

help() {
    awk -F'### ' '/^###/ { print $2 }' "$0"
}
if [[ $# == 0 ]] || [[ "$1" == "-h" ]] || [[ "$1" == "help" ]]; then
    help
    exit 1
fi

yes_or_no() {
    while true; do
        read -p "$* [y/n]: " yn
        case $yn in
            [Yy]*) return 0  ;;  
            [Nn]*) echo "Operation canceled." ; exit 0 ;;
        esac
    done
}

INPUT=$1

if [[ $2 == "" ]]; then
	yes_or_no "WARNING: Input will be overwritten. Continue?" && echo "Ok."
	OUTPUT="$INPUT"
else
	OUTPUT=$2
fi

# Calcolo header:
# Esclude tutte le linee con #, tre dash, minuscole, graffe aperte e spazi vuoti
HEADER=0
while read -r line; do
    if [[ "$line" == \#* ]] || [[ "$line" == \-\-\-* ]] || \
    [[ "$line" =~ ^[a-z] ]] || [[ "$line" == \{* ]] || [[ "$line" == "" ]]; then
            ((HEADER++))
    elif [[ "$line" =~ ^[A-Z] ]]; then
        break
    fi
done < <(head -n 20 "$INPUT")

cp "$INPUT" "$INPUT".orig

# Converte tre asterischi con tre caratteri U+02756
sed -i '' 's/^\#\# \*\*\*/\#\# \❖ \❖ \❖/g' "$INPUT"

# Calcolo capitoli:
# 0=con capitoli H2 - 1= senza capitoli
grep -q -E ^\#\# "$INPUT"
CAPITOLI=$?

# Rimuove linee con solo spazi vuote consecutive
sed -i '' -E -e '/^[[:space:]]$/d' -e '$!N;/^\n$/{$q;D;};P;D;' "$INPUT"

# Unifica i doppi apici
sed -i '' 's/[“”"]/\\"/g' "$INPUT"

# Inseriesce il shortcode "p" ovunque
tail -n "+$HEADER" "$INPUT" | sed '/^[-#{]/ ! s/^.*$/{{< p "&" >}}/' > "$OUTPUT".temp
sed -i '' 's/{{< p \"\" >}}//g' $OUTPUT.temp

# Fix per le interruzioni di pagina ___
sed -i '' 's/{{< p \"___\" >}}/___/g' $OUTPUT.temp

head -n "$HEADER" "$INPUT".orig > "$OUTPUT"

# NON SO perché l'inizio di OUTPUT.temp ha una riga vuota che dà un sacco di fastidio
tail -n +2 "$OUTPUT".temp >> "$OUTPUT"

rm "$OUTPUT".temp

if [[ ! -z $2 ]]; then 
	mv "$INPUT".orig "$INPUT"
else
	rm "$INPUT".orig
fi